package edu.sc.seis.gee.task;

import java.awt.BorderLayout;
import java.io.IOException;
import java.net.MalformedURLException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Set;
import java.util.StringTokenizer;
import java.util.regex.Pattern;
import javax.swing.JComponent;
import javax.swing.JPanel;
import javax.swing.ListSelectionModel;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import org.apache.log4j.Category;
import edu.iris.Fissures.IfEvent.EventAccessOperations;
import edu.iris.Fissures.IfEvent.NoPreferredOrigin;
import edu.iris.Fissures.IfNetwork.NetworkDCOperations;
import edu.sc.seis.fissuresUtil.cache.AbstractJob;
import edu.sc.seis.fissuresUtil.cache.BulletproofVestFactory;
import edu.sc.seis.fissuresUtil.cache.FilterNetworkDC;
import edu.sc.seis.fissuresUtil.cache.JobTracker;
import edu.sc.seis.fissuresUtil.cache.ProxyNetworkDC;
import edu.sc.seis.fissuresUtil.cache.VestingNetworkDC;
import edu.sc.seis.fissuresUtil.chooser.ChannelChooser;
import edu.sc.seis.fissuresUtil.exceptionHandler.GlobalExceptionHandler;
import edu.sc.seis.fissuresUtil.namingService.FissuresNamingService;
import edu.sc.seis.gee.CommonAccess;
import edu.sc.seis.gee.NetworkGateKeeper;
import edu.sc.seis.gee.NoCorbaException;
import edu.sc.seis.gee.NoNetworkException;
import edu.sc.seis.gee.configurator.ConfigurationException;

/**
 * ChannelChooserTask.java Created: Thu Feb 14 12:49:00 2002
 * 
 * @author <a href="mailto:">Srinivasa Telukutla </a>
 * @version $Id: ChannelChooserTask.java 15327 2005-11-28 21:35:28Z groves $
 */
public class ChannelChooserTask implements GUITask {

    /**
     * Gets the "More Options" GUI component.
     */
    public JComponent getMoreOptionsGUI() {
        return null;
    }

    /**
     * True if this GUI has a "more options" funtionality. More options appear
     * in a separate panel below the main gui, and can be shown/hidden with a
     * "Show More Options" and "Hide More Options" button that is provided
     * automatically is this return true.
     */
    public boolean hasMoreOptions() {
        return false;
    }

    public void invoke() {}

    /**
     * Gets the GUI for this Task. Used for interacting with the user before and
     * after invoking this Task's action.
     */
    public JComponent getGUI() throws Exception {
        if(!guiInitialized) {
            panel = new JPanel();
            guiInitialized = true;
            panel.setLayout(new BorderLayout());
            panel.add(getChannelChooser());
        }
        return panel;
    }

    public void configure(java.util.Map map) throws ConfigurationException {
        this.params = map;
        if(map.containsKey("showSites")
                && ((String)map.get("showSites")).equalsIgnoreCase(TRUE)) {
            showSites = true;
        } // end of if (map.containsKey("showSites"))
        if(map.containsKey("showNetworks")
                && ((String)map.get("showNetworks")).equalsIgnoreCase(FALSE)) {
            showNetworks = false;
        } // end of if (map.containsKey("showSites"))
        if(map.containsKey("showCodes")
                && ((String)map.get("showCodes")).equalsIgnoreCase(TRUE)) {
            showCodes = true;
        } // end of if (map.containsKey("showSites"))
        if(map.containsKey("eventSelector")) {
            TaskAction taskAction = CommonAccess.getCommonAccess()
                    .getTaskAction((String)map.get("eventSelector"));
            final ShowEventsTask showEventsTask = (ShowEventsTask)taskAction.getTask();
            final ListSelectionModel selectionModel = showEventsTask.getEventSelectionModel();
            selectionModel.addListSelectionListener(new ListSelectionListener() {

                public void valueChanged(ListSelectionEvent e) {
                    EventAccessOperations[] selectedEvents = showEventsTask.getSelectedEvents();
                    if(selectedEvents.length > 0) {
                        try {
                            // FIXME: make this work for more than one event
                            getChannelChooser().setAvailbleDataOrigin(selectedEvents[0].get_preferred_origin());
                        } catch(NoPreferredOrigin ex) {
                            // oh well
                        } catch(NoNetworkException ex) {
                            // probably can't happen as the event is over the
                            // net, but...
                            GlobalExceptionHandler.handle(ex);
                        }
                    }
                }
            });
        }
        if(map.containsKey("filterStations")) {
            String filterURL = (String)map.get("filterStations");
            try {
                patterns = FilterNetworkDC.readPattern(filterURL);
            } catch(MalformedURLException e) {
                throw new ConfigurationException("The url for the file with a list of stations to filter, "
                                                         + filterURL
                                                         + ", is malformed",
                                                 e);
            } catch(IOException e) {
                GlobalExceptionHandler.handle("Trouble reading list of patterns",
                                              e);
            }
        }
        if(map.containsKey("orientation")) {
            String orient = (String)map.get("orientation");
            if(orient.equals("INDIVIDUAL_CHANNELS")) {
                orientation = ChannelChooser.INDIVIDUAL_CHANNELS;
            } else if(orient.equals("THREE_COMPONENT")) {
                orientation = ChannelChooser.THREE_COMPONENT;
            } else if(orient.equals("VERTICAL_ONLY")) {
                orientation = ChannelChooser.VERTICAL_ONLY;
            } else if(orient.equals("HORIZONTAL_ONLY")) {
                orientation = ChannelChooser.HORIZONTAL_ONLY;
            } else {
                throw new ConfigurationException("That orientation string isn't supported.  As of now, INDIVIDUAL_CHANNELS is the only one.  Add more in ChannelChooserTask if you need it.");
            }
        }
        if(map.containsKey("bands")) {
            List bands = new ArrayList();
            StringTokenizer tokenizer = new StringTokenizer((String)map.get("bands"),
                                                            ",");
            while(tokenizer.hasMoreTokens()) {
                bands.add(tokenizer.nextToken());
            }
            autoSelectBand = (String[])bands.toArray(new String[0]);
        }
        // start initializing channel chooser in background
        initializeChannelChooser();
    }

    private Pattern[] patterns = new Pattern[0];

    public void destroy() {}

    protected VestingNetworkDC[] getConfiguredNetworkDCs()
            throws ConfigurationException {
        CommonAccess commonAccess = CommonAccess.getCommonAccess();
        FissuresNamingService fissuresNamingService = commonAccess.getFissuresNamingService();
        Set keySet = params.keySet();
        Iterator iter = keySet.iterator();
        ArrayList list = new ArrayList();
        while(iter.hasNext()) {
            String key = (String)iter.next();
            if(key.startsWith(SERVER_DNS)) {
                String dnsValue = (String)params.get(key);
                String nameValue = (String)params.get(SERVER_NAME
                        + key.substring(9));
                logger.debug(key + " " + key.substring(9) + " " + nameValue);
                if(dnsValue == null && dnsValue.length() == 0
                        && nameValue == null && nameValue.length() == 0) {
                    throw new ConfigurationException("Bad configuration for "
                            + key);
                } // end of if ()
                try {
                    VestingNetworkDC dc = new VestingNetworkDC(dnsValue,
                                                               nameValue,
                                                               fissuresNamingService);
                    if(patterns.length > 0) {
                        list.add(new FilterNetworkDC(dc, patterns));
                    } else {
                        list.add(dc);
                    }
                } catch(Exception e) {
                    throw new ConfigurationException("Unable to resolve NetworkDC:"
                                                             + dnsValue
                                                             + " "
                                                             + nameValue
                                                             + " FVer="
                                                             + edu.iris.Fissures.VERSION.value,
                                                     e);
                }
            }
        }
        return (VestingNetworkDC[])list.toArray(new VestingNetworkDC[list.size()]);
    }

    public String[] getConfiguredNetworks() {
        Set keySet = params.keySet();
        Iterator iter = keySet.iterator();
        ArrayList arrayList = new ArrayList();
        while(iter.hasNext()) {
            String key = (String)iter.next();
            if(key.startsWith("network")) {
                String value = (String)params.get(key);
                arrayList.add(value);
            }
        }
        String[] rtnValues = new String[arrayList.size()];
        rtnValues = (String[])arrayList.toArray(rtnValues);
        return rtnValues;
    }

    /**
     * This acts a a cubby hole, allowing the channel chooser initializer thread
     * to safely create the channel chooser in a separate thread, and then give
     * to the caller with a notify.
     */
    public ChannelChooser getChannelChooser() throws NoNetworkException {
        if(!NetworkGateKeeper.corbaAccessAllowed()) {
            throw new NoCorbaException();
        }
        if(channelChooserCubbyHole == null) {
            try {
                initializeChannelChooser();
            } catch(ConfigurationException e) {
                GlobalExceptionHandler.handle("Problem initializing the channel chooser",
                                              e);
            }
        }
        // this get() will block until the channel chooser is initialized
        return channelChooserCubbyHole.get();
    }

    protected void initializeChannelChooser() throws ConfigurationException {
        if(channelChooserCubbyHole == null) {
            channelChooserCubbyHole = new ChannelChooserCubbyHole();
            ChannelChooserInitializer cci = new ChannelChooserInitializer();
            JobTracker.getTracker().add(cci);
            if(processDisplayAction != null) {
                processDisplayAction.checkDisplayLocation();
            }
            cci.run();
        }
    }

    private class ChannelChooserCubbyHole {

        private ChannelChooser contents;

        private boolean available = false;

        public synchronized ChannelChooser get() {
            if(available == false) {
                logger.debug("Waiting to get channel chooser from CubbyHole");
            }
            while(available == false) {
                try {
                    wait();
                } catch(InterruptedException e) {}
            }
            notifyAll();
            return contents;
        }

        public synchronized void put(ChannelChooser value) {
            while(available == true) {
                try {
                    wait();
                } catch(InterruptedException e) {}
            }
            contents = value;
            available = true;
            notifyAll();
        }
    }

    private String[] autoSelectBand = {"B"};

    private int orientation = ChannelChooser.BEST_CHANNELS;

    private class ChannelChooserInitializer extends AbstractJob {

        public ChannelChooserInitializer() {
            super("Channel Chooser");
        }

        public void runJob() {
            if(!NetworkGateKeeper.corbaAccessAllowed()) {
                setFinished();
                return;
            }
            VestingNetworkDC[] netDCs = null;
            try {
                setStatus("Getting Networks");
                netDCs = getConfiguredNetworkDCs();
            } catch(ConfigurationException e) {}
            String[] band = {"B", "L", "S", "E"};
            setStatus("Creating chooser");
            ChannelChooser myChannelChooser = new ChannelChooser(netDCs,
                                                                 showSites,
                                                                 showNetworks,
                                                                 getConfiguredNetworks(),
                                                                 band,
                                                                 autoSelectBand,
                                                                 ChannelChooser.DEFAULT_SELECTABLE_ORIENTATIONS,
                                                                 orientation);
            if(showCodes) {
                myChannelChooser.setShowCodes(showCodes);
            }
            String dcTaskId = (String)params.get("loadSeismogramTask");
            setStatus("Getting Seismogram Loaders");
            if(dcTaskId != null) {
                TaskAction taskAction = CommonAccess.getCommonAccess()
                        .getTaskAction(dcTaskId);
                try {
                    LoadSeismogram loadSeismogramTask = (LoadSeismogram)taskAction.getTask();
                    myChannelChooser.setSeismogramDC(loadSeismogramTask.getDataCenterRouter());
                } catch(Exception e) {
                    GlobalExceptionHandler.handle("Unable to get seismogram DC from task "
                                                          + dcTaskId,
                                                  e);
                }
            }
            channelChooserCubbyHole.put(myChannelChooser);
            setFinished();
        }
    }

    java.util.Map params = new HashMap();

    JPanel panel;

    boolean showSites = false;

    boolean showNetworks = true;

    boolean showCodes = false;

    protected boolean guiInitialized = false;

    private ChannelChooserCubbyHole channelChooserCubbyHole = null;

    static Category logger = Category.getInstance(ChannelChooserTask.class.getName());

    private TaskAction processDisplayAction;
}// ChannelChooserTask
